class ApplicationController < ActionController::Base

  helper :all
  protect_from_forgery
  filter_parameter_logging :password
  layout "application"

  def authenticate
    if session[:user].nil? or session[:password].nil? or
       (user = read_users[session[:user]]).nil? or
       RailsSvnAdmin::Htpasswd.md5(session[:password], user[:salt]) != user[:md5]
      flash[:error] = t(:error_require_authentication)
      redirect_to(:controller => :users, :action => :login, :back_url => url_for(params))
    end
  end

  def require_admin
    if !current_user.admin?
      flash[:error] = t(:error_require_admin)
      redirect_to(:controller => :users, :action => :login, :back_url => url_for(params))
    end
  end

  def current_user
    @current_user ||= User.new(session)
  end
  helper_method :current_user

protected

  def read_users
    users = {}
    if File.exists?(Setting[:password_file]) and File.file?(Setting[:password_file]) and File.readable?(Setting[:password_file])
      file = File.open(Setting[:password_file], "r")
      file.read.split("\n").each do |line|
        user = RailsSvnAdmin::Htpasswd.decode(line)
        users[user[:user]] = user
      end
      file.close
    end
    users
  end

  def write_users(users)
    if File.exists?(Setting[:password_file]) and File.file?(Setting[:password_file]) and File.writable?(Setting[:password_file])
      file = File.open(Setting[:password_file], "w")
      file.write(users.values.collect{|user| RailsSvnAdmin::Htpasswd.encode(user)}.join("\n"))
      file.close
    end
  end

end
