require "htauth/md5"
require "rails_svn_admin/htpasswd"

class UsersController < ApplicationController

  before_filter :authenticate, :except => [:login]
  before_filter :require_admin, :except => [:login, :logout, :change_my_password]

  def login
    @back_url = params[:back_url]
    if request.post?
      if !File.exists?(Setting[:password_file])
        flash[:error] = t(:error_password_file_not_found)
      else
        file = File.open(Setting[:password_file], "r")
        found = nil
        while found.nil? and (line = file.gets)
          values = RailsSvnAdmin::Htpasswd::decode(line)
          if params[:user] == values[:user]
            found = values.clone
          end
        end
        file.close
        if found.nil?
          flash[:error] = t(:error_user_not_found, :user => params[:user])
        elsif RailsSvnAdmin::Htpasswd::md5(params[:password], found[:salt]) != found[:md5]
          flash[:error] = t(:error_invalid_credentials, :user => params[:user])
        else
          flash[:notice] = t(:notice_successfully_login)
          session[:user] = params[:user]
          session[:password] = params[:password]
          if current_user.admin?
            redirect_to({:action => :index})
          else
            redirect_to({:action => :change_my_password, :user => session[:user]})
          end
        end
      end
    end
  end

  def logout
    session[:user] = nil
    session[:password] = nil
    redirect_to(:action => :login)
  end

  def index
    @users = read_users.values.sort{|a, b| a[:user].downcase <=> b[:user].downcase}
  end

  def add
    if request.post?
      users = read_users
      if params[:password] != params[:password_repeat]
        flash[:error] = t(:error_passwords_different)
      elsif users.include?(params[:user])
        flash[:error] = t(:error_user_exists, :user => params[:user])
      else
        user = RailsSvnAdmin::Htpasswd::htpasswd(params[:user], params[:password])
        users[params[:user]] = user
        write_users(users)
        flash[:notice] = t(:notice_user_edited)
      end
      redirect_to({:action => :index})
    end
  end

  def edit
    users = read_users
    @user = users[params[:user]]
    if request.post?
      changing_password = !params[:password].empty? or !params[:password_repeat].empty?
      if changing_password and params[:password] != params[:password_repeat]
        flash[:error] = t(:error_passwords_different)
      elsif @user[:user] != params[:user_new] and users.include?(params[:user_new])
        flash[:error] = t(:error_user_exists, :user => params[:user_new])
      else
        @user[:user] = params[:user_new]
        @user[:md5] = RailsSvnAdmin::Htpasswd::md5(params[:password], @user[:salt]) if changing_password
        write_users(users)
        flash[:notice] = t(:notice_user_edited)
      end
      redirect_to({:action => :index})
    end
  end

  def change_my_password
    if session[:user] != params[:user]
      flash[:error] = t(:error_invalid_user_to_change_password, :user => params[:user])
      redirect_to({:user => session[:user]})
    else
      users = read_users
      @user = users[params[:user]]
      if request.post?
        if @user[:md5] != RailsSvnAdmin::Htpasswd::md5(params[:password_previous], @user[:salt])
          flash[:error] = t(:error_invalid_credentials, :user => params[:user])
        elsif params[:password] != params[:password_repeat]
          flash[:error] = t(:error_passwords_different)
        else
          @user[:md5] = RailsSvnAdmin::Htpasswd::md5(params[:password], @user[:salt])
          session[:password] = params[:password]
          write_users(users)
          flash[:notice] = t(:notice_user_edited)
        end
        if current_user.admin?
          redirect_to({:action => :index})
        else
          redirect_to({:action => :change_my_password, :user => current_user.user})
        end
      end
    end
  end

  def delete
    users = read_users
    if !users.nil? and !users.empty?
      new_users = {}
      users.values.each do |user|
        new_users[user[:user]] = user if user[:user] != params[:user]
      end
      write_users(new_users)
    end
    redirect_to({:action => :index})
  end

end
